@@ -12,8 +12,14 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: boxOp.c,v 1.3 1996/04/19 09:05:53 torsten Exp $ */
+/* $Id: boxOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtWindow XTWINDOW
+#endif
+
+#include <math.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include "xpaint.h"
@@ -32,7 +38,7 @@ typedef struct {
     GC gcx;
 } LocalInfo;
 
-#define MKRECT(rect, sx, sy, ex, ey, typeFlag) do {			\
+#define MKRECT(rect, sx, sy, ex, ey, typeFlag) {			\
 	if (typeFlag == 1) {	/* square */				\
 		(rect)->width  = MAX(ABS(sx - ex),ABS(sy - ey));	\
 		(rect)->height = (rect)->width;				\
@@ -55,9 +61,11 @@ typedef struct {
 		(rect)->width  = MAX(sx, ex) - (rect)->x;		\
 		(rect)->height = MAX(sy, ey) - (rect)->y;		\
 	}								\
-} while (0)
+}
 
 static int boxStyle = 0;
+static int boxSize = 7;
+static float boxRatio = 0.0;
 
 static void 
 press(Widget w, LocalInfo * l,
@@ -66,6 +74,7 @@ press(Widget w, LocalInfo * l,
     /*
     **	Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;   
     if ((event->state & AllButtonsMask) != 0)
 	return;
     if (event->button == Button3) return;
@@ -79,10 +88,52 @@ press(Widget w, LocalInfo * l,
     SetCapAndJoin(w, info->first_gc, JoinMiter);
     SetCapAndJoin(w, info->second_gc, JoinMiter);
 
-    l->type = 2 * boxStyle + ((event->state & ShiftMask) ? 1 : 0);
+    l->type = 2 * (boxStyle & 1) + ((event->state & ShiftMask) ? 1 : 0);
     l->drawn = False;
 }
 
+void
+DrawBox(Display *dpy, Drawable d, GC gc, XRectangle *rect, Boolean fill)
+{
+    XPoint *xp;
+    int i, n, p;
+    float a, r, c, s;
+
+    if ((boxStyle & 2) == 0 || (boxSize == 0 && boxRatio == 0.0)) {
+        if (fill)
+            XFillRectangles(dpy, d, gc, rect, 1);
+        else
+            XDrawRectangles(dpy, d, gc, rect, 1);
+    } else {
+        if (boxSize>0)
+	    r = MIN(boxSize, MIN(rect->width, rect->height)/2);
+        else
+            r = 0.5 * MIN(rect->width, rect->height) * boxRatio;
+        p = 3 + 4.0 * sqrt(r);
+        n = 4 * p + 1;
+        xp = (XPoint *) malloc(n * sizeof(XPoint));
+        for (i=0; i<p; i++) {
+  	    a = i*M_PI/(2*p-2);
+            c = 1.0 - cos(a);
+            s = 1.0 - sin(a);
+            xp[i].x = rect->x + r * c + 0.5;
+            xp[i].y = rect->y + r * s + 0.5;
+            xp[i+p].x = rect->x + rect->width - r * s + 0.5;
+            xp[i+p].y = rect->y + r * c + 0.5;
+            xp[i+2*p].x = rect->x + rect->width - r * c + 0.5;
+            xp[i+2*p].y = rect->y + rect->height - r * s + 0.5;
+            xp[i+3*p].x = rect->x + r * s + 0.5;
+            xp[i+3*p].y = rect->y + rect->height - r * c + 0.5;
+	}
+        xp[n-1] = xp[0];
+        if (fill)
+            XFillPolygon(dpy, d, gc, xp, n, Complex, CoordModeOrigin);
+        else
+            XDrawLines(dpy, d, gc, xp, n, CoordModeOrigin);
+        free(xp);
+    }
+}
+
 static void 
 motion(Widget w, LocalInfo * l,
        XMotionEvent * event, OpInfo * info)
@@ -94,18 +145,16 @@ motion(Widget w, LocalInfo * l,
 
     if (l->drawn) {
 	MKRECT(&rect, l->startX, l->startY, l->endX, l->endY, l->type);
-
-	XDrawRectangles(XtDisplay(w), XtWindow(w), l->gcx, &rect, 1);
+        DrawBox(XtDisplay(w), XtWindow(w), l->gcx, &rect, 0);
     }
     l->endX = event->x;
     l->endY = event->y;
 
-    l->type = 2 * boxStyle + ((event->state & ShiftMask) ? 1 : 0);
+    l->type = 2 * (boxStyle & 1) + ((event->state & ShiftMask) ? 1 : 0);
 
     if ((l->drawn = (l->startX != l->endX || l->startY != l->endY))) {
 	MKRECT(&rect, l->startX, l->startY, l->endX, l->endY, l->type);
-
-	XDrawRectangles(XtDisplay(w), XtWindow(w), l->gcx, &rect, 1);
+        DrawBox(XtDisplay(w), XtWindow(w), l->gcx, &rect, 0);
     }
 }
 
@@ -120,6 +169,7 @@ release(Widget w, LocalInfo * l,
     /*
     **	Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;
     mask = AllButtonsMask;
     switch (event->button) {
     case Button1:
@@ -144,9 +194,9 @@ release(Widget w, LocalInfo * l,
 
     if (l->drawn && info->surface == opWindow) {
 	MKRECT(&rect, l->startX, l->startY, l->endX, l->endY, l->type);
-
-	XDrawRectangles(XtDisplay(w), XtWindow(w), l->gcx, &rect, 1);
+        DrawBox(XtDisplay(w), XtWindow(w), l->gcx, &rect, 0);
     }
+
     if (info->isFat && info->surface == opWindow)
 	return;
 
@@ -163,8 +213,9 @@ release(Widget w, LocalInfo * l,
     }
 
     if (l->fill)
-	XFillRectangles(XtDisplay(w), info->drawable, fgc, &rect, 1);
-    XDrawRectangles(XtDisplay(w), info->drawable, lgc, &rect, 1);
+	DrawBox(XtDisplay(w), info->drawable, fgc, &rect, 1);
+    DrawBox(XtDisplay(w), info->drawable, lgc, &rect, 0);
+
 
     if (info->surface == opPixmap) {
 	rect.width++;
@@ -215,7 +266,7 @@ BoxRemove(Widget w, void *l)
 }
 
 void *
-FBoxAdd(Widget w)
+FilledBoxAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
 
@@ -237,7 +288,7 @@ FBoxAdd(Widget w)
 }
 
 void 
-FBoxRemove(Widget w, void *l)
+FilledBoxRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opWindow, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
@@ -250,13 +301,20 @@ FBoxRemove(Widget w, void *l)
 }
 
 void 
-BoxSetStyle(Boolean mode)
+BoxSetStyle(int mode)
 {
-    boxStyle = mode ? 1 : 0;
+    boxStyle = mode;
 }
 
-Boolean
+int
 BoxGetStyle(void)
 {
-    return boxStyle == 0 ? False : True;
+    return boxStyle;
+}
+
+void 
+BoxSetParameter(int b_size, float b_ratio)
+{
+    boxSize = b_size;
+    boxRatio = b_ratio;
 }
