@@ -13,7 +13,11 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: circleOp.c,v 1.3 1996/04/19 09:06:03 torsten Exp $ */
+/* $Id: circleOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -91,6 +95,7 @@ press(Widget w, LocalInfo * l,
     /*
     **	Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;      
     if ((event->state & AllButtonsMask) != 0)
 	return;
     if (event->button == Button3) return;
@@ -141,6 +146,7 @@ release(Widget w, LocalInfo * l,
     /*
     **	Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;
     mask = AllButtonsMask;
     switch (event->button) {
     case Button1:
@@ -231,7 +237,7 @@ CircleRemove(Widget w, void *l)
 }
 
 void *
-FCircleAdd(Widget w)
+FilledCircleAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->flag = CIRCLE | FILL;
@@ -249,7 +255,7 @@ FCircleAdd(Widget w)
 }
 
 void 
-FCircleRemove(Widget w, void *l)
+FilledCircleRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opWindow, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
@@ -291,7 +297,7 @@ OvalRemove(Widget w, void *l)
 }
 
 void *
-FOvalAdd(Widget w)
+FilledOvalAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->flag = FILL;
@@ -310,7 +316,7 @@ FOvalAdd(Widget w)
 }
 
 void 
-FOvalRemove(Widget w, void *l)
+FilledOvalRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opWindow, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
