@@ -13,21 +13,22 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fileName.c,v 1.12 2000/10/02 20:55:17 torsten Exp $ */
+/* $Id: fileName.c,v 1.18 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Dialog.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Xaw/List.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Text.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Scrollbar.h>
+
+#include "xaw_incdir/Dialog.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Label.h"
+#include "xaw_incdir/List.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Text.h"
+#include "xaw_incdir/Viewport.h"
+#include "xaw_incdir/Scrollbar.h"
 
 #include "Paint.h"
 #include "PaintP.h"
@@ -38,7 +39,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if defined(SYSV) || defined(SVR4) || defined(__CYGWIN__)
+#if defined(SYSV) || defined(SVR4) || defined(__CYGWIN__) || defined(__VMS )
 #include <dirent.h>
 #else
 #include <sys/dir.h>
@@ -87,18 +88,14 @@ GetFileNameGetLastId()
  * This function is called by all image save functions.
  * If 'flag' is True, save the entire image; otherwise, save the region.
  */
-static void 
-stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
+static void
+okSaveCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
 {
     Pixmap pix, mask = None;
     int width, height;
     Colormap cmap;
     Image *image;
 
-    if (*file == '\0') {
-	Notice(paint, msgText[NO_FILE_NAME_SUPPLIED]);
-	return;
-    }
     StateSetBusy(True);
     if (flag) {
 	PwGetPixmap(paint, &pix, &width, &height);
@@ -136,6 +133,40 @@ stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
     ImageDelete(image);
     StateSetBusy(False);
 }
+static void
+okOverwriteCallback(Widget shell, XtPointer data, XtPointer junk)
+{
+    void **ptr = data;
+    okSaveCallback(shell, (char *)ptr[0], ptr[1]?True:False, 
+                   (RWwriteFunc)ptr[2]);
+}
+static void
+nokOverwriteCallback(Widget shell, XtPointer data, XtPointer junk)
+{
+}
+
+static void 
+stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
+{
+    char msg[512];
+    static void *ptr[3];
+    struct stat buf;
+
+    if (*file == '\0') {
+	Notice(paint, msgText[NO_FILE_NAME_SUPPLIED]);
+	return;
+    }
+    if (!stat(file, &buf)) {
+        sprintf(msg, msgText[OVERWRITE_FILE], file);
+	ptr[0] = file;
+	ptr[1] = (void *)((int)flag);
+	ptr[2] = (void *)f;
+        AlertBox(paint, msg, okOverwriteCallback, 
+                             nokOverwriteCallback, (XtPointer)ptr);
+	return;
+    }
+    okSaveCallback(paint, file, flag, f);
+}
 
 static void 
 saveRegionFileCallback(Widget paint, XtPointer str, XtPointer func)
@@ -232,16 +263,18 @@ typedef struct arg_s {
     XtCallbackProc okFunc;
     void *type;
     Boolean isRead, isSimple;
-    Widget w, text, scrollbar;
+    Widget w;
     XtPointer closure;
     char dirname[MAX_PATH];
-    Widget list, cwd_w, info;
-    int first;
+    Widget shell, parent, browser,
+           form, title_w, name, vport, list, cwd_w, info,
+           ok, cancel;
+    Widget format[NUM_FORMATS+2];
+    int first, numf, oldwidth;
     Boolean isPoped;
     /*
     **	Use for caching, list purposes
      */
-    Widget parent;
     int browserType;
     struct arg_s *next;
 } arg_t;
@@ -272,6 +305,49 @@ emptyList(arg_t * arg)
     XtFree((XtPointer) strs);
 }
 
+static void
+browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height, width2;
+    Position x, y;
+    int i, width1, height1, dw = 0;
+
+    XtVaGetValues(XtParent(l->parent), 
+		  XtNwidth, &width, XtNheight, &height, NULL);
+    if (l->oldwidth!= 0) dw = (int)width - l->oldwidth;   
+    l->oldwidth = (int)width;
+    XtVaGetValues(l->ok, XtNwidth, &width2, NULL);
+    i = width2;
+    XtVaGetValues(l->cancel, XtNwidth, &width2, NULL);
+    i += width2 + 35;
+    width1 = (int)width - i; if (width1<60) width1 = 60;
+    height1 = (int)height - 120; if (height1<60) height1 = 60;
+
+    XtResizeWidget(l->browser, width, height, 0);
+    XtResizeWidget(l->parent, width, height, 0);   
+    XtVaSetValues(l->browser, XtNwidth, width, XtNheight, height, NULL);
+    XtVaSetValues(l->parent, XtNwidth, width, XtNheight, height, NULL);   
+    XtResizeWidget(l->name, width1-2, 20, 1);
+    XtResizeWidget(l->form, width1 + 10, height, 0);
+    XtVaGetValues(l->list, XtNlongest, &width2, NULL);
+    XtVaSetValues(l->list, XtNdefaultColumns, 1, NULL);
+    XtVaSetValues(l->list, XtNwidth, 
+		  (width2>width1-30)?width2+10:width1-20, NULL);
+    XtResizeWidget(l->vport, width1, height1, 1);     
+    XtVaSetValues(l->vport, XtNwidth, width1, XtNheight, height1, NULL);
+    XtVaGetValues(l->ok, XtNx, &x, XtNy, &y, NULL);
+    if (dw) XtMoveWidget(l->ok, x+dw, y);
+    XtVaGetValues(l->cancel, XtNx, &x, XtNy, &y, NULL);  
+    if (dw) XtMoveWidget(l->cancel, x+dw, y);   
+    if (dw) for (i=0; i<l->numf; i++) {
+        XtVaGetValues(l->format[i], XtNx, &x, XtNy, &y, NULL);  
+        XtMoveWidget(l->format[i], x+dw, y);   
+    }
+    XtVaGetValues(l->cwd_w, XtNx, &x, NULL);
+    XtMoveWidget(l->cwd_w, x, height - 57);
+    XtMoveWidget(l->info, x, height - 30);
+}
+
 static char *
 doDirname(arg_t * arg, char *file)
 {
@@ -311,8 +387,10 @@ doDirname(arg_t * arg, char *file)
     } else {
 	strcpy(newPath, arg->dirname);
     }
+#ifndef __VMS
     if (strcmp(newPath, "/") != 0)
 	strcat(newPath, "/");
+#endif
     while (*file != '\0') {
 	char *ep;
 
@@ -463,12 +541,11 @@ setCWD(arg_t * arg, char *dir)
 	XtVaSetValues(arg->cwd_w, XtNlabel, dir, NULL);
     }
     XawListChange(arg->list, list, i, 0, True);
-    XtVaSetValues(arg->text, XtNstring, "", NULL);
+    XtVaSetValues(arg->name, XtNstring, "", NULL);
     sprintf(infoStr, "%d %s, %d %s", 
         dirCount, msgText[DIRECTORIES], fileCount, msgText[FILES]);
     XtVaSetValues(arg->info, XtNlabel, infoStr, NULL);
-
-    if ((sb = XtNameToWidget(arg->scrollbar, "vertical")) != None) {
+    if ((sb = XtNameToWidget(arg->vport, "vertical")) != None) {
 	float top = 0.0;
 
 	XawScrollbarSetThumb(sb, top, -1.0);
@@ -488,7 +565,7 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     char *cp;
     char *nm;
 
-    XtVaGetValues(arg->text, XtNstring, &str, NULL);
+    XtVaGetValues(arg->name, XtNstring, &str, NULL);
     if (str == NULL || *str == '\0') {
 	XawListReturnStruct *lr = XawListShowCurrent(arg->list);
 
@@ -506,15 +583,19 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     }
     if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0) {
 	setCWD(arg, file);
+        browserResized(arg->parent, arg, NULL, False); 
 	return;
     }
     if ((cp = strrchr(file, '/')) != NULL) {
 	*cp = '\0';
-	if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0)
+	if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0) {
 	    setCWD(arg, file);
+            browserResized(arg->parent, arg, NULL, False);
+	}
 	*cp = '/';
     }
     arg->isPoped = False;
+    arg->oldwidth = 0;
     XtPopdown(GetShell(w));
 
     if (arg->isSimple) {
@@ -553,6 +634,7 @@ cancelCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
     arg_t *arg = (arg_t *) argArg;
     arg->isPoped = False;
+    arg->oldwidth = 0;   
     XtPopdown(GetShell(w));
 }
 
@@ -568,14 +650,14 @@ listCallback(Widget bar, XtPointer argArg, XtPointer itemArg)
     else
 	label = item->string;
 
-    XtVaGetValues(arg->text, XtNstring, &str, NULL);
+    XtVaGetValues(arg->name, XtNstring, &str, NULL);
 
     if (strcmp(str, label) == 0)
 	okCallback(bar, argArg, NULL);
     else
-	XtVaSetValues(arg->text, XtNstring, label, NULL);
+	XtVaSetValues(arg->name, XtNstring, label, NULL);
 }
-
+   
 static Widget
 buildBrowser(Widget parent, char *title, arg_t * arg)
 {
@@ -584,6 +666,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
     Widget form;
     Widget title_w, name, list, vport, cwd, info;
 
+    arg->parent = parent;
+
     if (!inited) {
 	inited = True;
 
@@ -607,6 +691,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				   XtNleft, XtChainLeft,
 				   XtNtop, XtChainTop,
 				   NULL);
+    arg->form = form;
 
     title_w = XtVaCreateManagedWidget("title",
 				      labelWidgetClass, form,
@@ -616,6 +701,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				      XtNbottom, XtChainTop,
 				      XtNjustify, XtJustifyLeft,
 				      NULL);
+    arg->title_w = title_w;
 
     name = XtVaCreateManagedWidget("name",
 				   asciiTextWidgetClass, form,
@@ -628,6 +714,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				   XtNtop, XtChainTop,
 				   XtNwidth, 250,
 				   NULL);
+    arg->name = name;
 
     vport = XtVaCreateManagedWidget("vport",
 				    viewportWidgetClass, form,
@@ -639,17 +726,22 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
 				    XtNwidth, 250,
-				    XtNheight, 200,
+				    XtNheight, 380,
 				    NULL);
+    arg->vport = vport;
 
     list = XtVaCreateManagedWidget("files",
 				   listWidgetClass, vport,
 				   XtNborderWidth, 1,
 				   XtNverticalList, True,
+				   XtNforceBars, False,
+/*				   
 				   XtNforceColumns, True,
 				   XtNdefaultColumns, 1,
+ */
 				   XtNnumberStrings, 0,
 				   NULL);
+    arg->list = list;
 
     cwd = XtVaCreateManagedWidget("cwd",
 				  labelWidgetClass, form,
@@ -661,6 +753,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				  XtNjustify, XtJustifyLeft,
 				  XtNresizable, True,
 				  NULL);
+    arg->cwd_w = cwd;
+   
     info = XtVaCreateManagedWidget("info",
 				   labelWidgetClass, form,
 				   XtNborderWidth, 0,
@@ -669,19 +763,15 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				   XtNbottom, XtChainBottom,
 				   XtNjustify, XtJustifyLeft,
 				   NULL);
+    arg->info = info;
 
     XtAddCallback(list, XtNcallback, listCallback, (XtPointer) arg);
-
-    arg->info = info;
-    arg->cwd_w = cwd;
-    arg->list = list;
-    arg->text = name;
-    arg->scrollbar = vport;
+    XtAddEventHandler(list, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
 
     arg->first = True;
 
     setCWD(arg, doDirname(arg, NULL));
-
     return form;
 }
 
@@ -692,21 +782,23 @@ buildOpenBrowser(Widget w, arg_t * arg)
     Widget toggle, firstToggle = None;
     XtAccelerators accel;
     XtTranslations toglt;
-    int i, j;
+    int i, j, k;
     char *name_format;
     char **list;
-    Arg args[3];
+    Arg args[4];
     int nargs = 0;
     
     
     shell = XtVisCreatePopupShell("filebrowser",
 				  transientShellWidgetClass, GetToplevel(w),
 				  args, nargs);
+    arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
     browser = buildBrowser(form, msgText[OPEN_FILE_NAMED], arg);
+    arg->browser = browser;
 
     accel = XtParseAcceleratorTable("#override\n\
 					 <Key>Return: set() notify() unset()\n\
@@ -720,6 +812,7 @@ buildOpenBrowser(Widget w, arg_t * arg)
 				       XtNleft, XtChainRight,
 				       XtNright, XtChainRight,
 				       NULL);
+    arg->ok = okButton;
     cancelButton = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, form,
 					   XtNfromHoriz, okButton,
@@ -727,6 +820,7 @@ buildOpenBrowser(Widget w, arg_t * arg)
 					   XtNleft, XtChainRight,
 					   XtNright, XtChainRight,
 					   NULL);
+    arg->cancel = cancelButton;
 
     toggle = okButton;
     toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
@@ -734,6 +828,7 @@ buildOpenBrowser(Widget w, arg_t * arg)
     list = RWtableGetReaderList();
 
     j = 0;
+    k = 0;
     for (i = 0; list[i] != NULL; i++) {
         while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))) {
 	    if (j<NUM_FORMATS) ++j;
@@ -744,6 +839,7 @@ buildOpenBrowser(Widget w, arg_t * arg)
 	else
 	   name_format = list[i];
 	if (j<NUM_FORMATS) ++j;
+
 	toggle = XtVaCreateManagedWidget(list[i],
 					 toggleWidgetClass, form,
 					 XtNtranslations, toglt,
@@ -756,11 +852,13 @@ buildOpenBrowser(Widget w, arg_t * arg)
 					 XtNleft, XtChainRight,
 					 XtNright, XtChainRight,
 					 NULL);
+        arg->format[k] = toggle;
+        k++;
+
 	if (firstToggle == None) {
 	    arg->type = NULL;
 	    XtVaSetValues(toggle, XtNstate, True,
-			  XtNvertDistance, 100,
-			  NULL);
+			  XtNvertDistance, 29, NULL);
 	    firstToggle = toggle;
 	}
 	XtAddCallback(toggle, XtNcallback, fileTypeCallback, (XtPointer) arg);
@@ -768,14 +866,15 @@ buildOpenBrowser(Widget w, arg_t * arg)
 
     arg->isSimple = False;
     arg->isRead = True;
+    arg->numf = k;
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->text);
-    XtInstallAccelerators(arg->text, okButton);
+    XtSetKeyboardFocus(form, arg->name);
+    XtInstallAccelerators(arg->name, okButton);
 
     return shell;
 }
@@ -786,22 +885,25 @@ buildSaveBrowser(Widget w, arg_t * arg)
     Widget shell, browser, okButton, cancelButton, form;
     Widget toggle, firstToggle = None;
     XtAccelerators accel;
+
     XtTranslations toglt;
-    int i, j;
+    int i, j, k;
     char *name_format;
     char **list, *rdr = NULL;
-    Arg args[3];
+    Arg args[4];
     int nargs=0;
     
     shell = XtVisCreatePopupShell("filebrowser",
 			       transientShellWidgetClass, GetToplevel(w),
 				args, nargs);
+    arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
     browser = buildBrowser(form, msgText[SAVE_IN_FILE], arg);
-
+    arg->browser = browser;
+   
     accel = XtParseAcceleratorTable("#override\n\
 					 <Key>Return: set() notify() unset()\n\
 					 <Key>Linefeed: set() notify() unset()");
@@ -812,11 +914,13 @@ buildSaveBrowser(Widget w, arg_t * arg)
 				       XtNaccelerators, accel,
 				       XtNbottom, XtChainTop,
 				       NULL);
+    arg->ok = okButton;      
     cancelButton = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, form,
 					   XtNfromHoriz, okButton,
 					   XtNbottom, XtChainTop,
 					   NULL);
+    arg->cancel = cancelButton;
 
     toggle = okButton;
     toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
@@ -827,6 +931,7 @@ buildSaveBrowser(Widget w, arg_t * arg)
     }
 
     j = 0;
+    k = 0;
     for (i = 0; list[i] != NULL; i++) {
         while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))) {
 	    if (j<NUM_FORMATS) ++j;
@@ -848,9 +953,12 @@ buildSaveBrowser(Widget w, arg_t * arg)
 					 XtNtop, XtChainBottom,
 					 XtNbottom, XtChainBottom,
 					 NULL);
+        arg->format[k] = toggle;
+        k++;
+
 	if (firstToggle == None) {
 	    arg->type = NULL;
-	    XtVaSetValues(toggle, XtNvertDistance, 100, NULL);
+	    XtVaSetValues(toggle, XtNvertDistance, 29, NULL);
 	    firstToggle = toggle;
 	    if (rdr == NULL)
 		XtVaSetValues(toggle, XtNstate, True, NULL);
@@ -864,14 +972,15 @@ buildSaveBrowser(Widget w, arg_t * arg)
 
     arg->isSimple = False;
     arg->isRead = False;
+    arg->numf = k;
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->text);
-    XtInstallAccelerators(arg->text, okButton);
+    XtSetKeyboardFocus(form, arg->name);
+    XtInstallAccelerators(arg->name, okButton);
 
     return shell;
 }
@@ -881,12 +990,13 @@ buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
 {
     Widget shell, browser, okButton, cancelButton, form;
     XtAccelerators accel;
-    Arg args[3];
+    Arg args[4];
     int nargs = 0;
 
     shell = XtVisCreatePopupShell("filebrowser",
 				  transientShellWidgetClass, GetToplevel(w),
 				  args, nargs);
+    arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
@@ -894,6 +1004,7 @@ buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
     browser = buildBrowser(form, isSave ? 
 			   msgText[SAVE_IN_FILE] : msgText[LOAD_FROM_FILE],
 			   arg);
+    arg->browser = browser;   
 
     accel = XtParseAcceleratorTable("#override\n\
 					 <Key>Return: set() notify() unset()\n\
@@ -905,21 +1016,24 @@ buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
 				       XtNaccelerators, accel,
 				       XtNbottom, XtChainTop,
 				       NULL);
+    arg->ok = okButton;
     cancelButton = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, form,
 					   XtNfromHoriz, okButton,
 					   XtNbottom, XtChainTop,
 					   NULL);
+    arg->cancel = cancelButton;   
 
     arg->isSimple = True;
+    arg->numf = 0;
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->text);
-    XtInstallAccelerators(arg->text, okButton);
+    XtSetKeyboardFocus(form, arg->name);
+    XtInstallAccelerators(arg->name, okButton);
 
     return shell;
 }
@@ -943,7 +1057,7 @@ freeArg(Widget w, arg_t * arg)
 
     *pp = arg->next;
 
-    XtDestroyWidget(GetShell(arg->text));
+    XtDestroyWidget(GetShell(arg->name));
     XtFree((XtPointer) arg);
 }
 
@@ -1014,13 +1128,15 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     arg_t *arg;
     Boolean built;
     Position x, y;
+    Widget shell;
+    int i;
 
     XtVaGetValues(GetShell(w), XtNx, &x, XtNy, &y, NULL);
 
     arg = getArg(w, type, &built);
     arg->closure = data;
 
-    XtVaSetValues(GetShell(arg->text), XtNx, x + 24, XtNy, y + 24, NULL);
+    XtVaSetValues(GetShell(arg->name), XtNx, x + 24, XtNy, y + 24, NULL);
 
     if (def != NULL && *def != '\0') {
 	char *cp, dirname[MAX_PATH];
@@ -1035,7 +1151,7 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
 	if (built | Global.explore)		/* This browser has not been used before, init it. */
 	    setCWD(arg, doDirname(arg, dirname));
 
-	XtVaSetValues(arg->text, XtNstring, cp,
+	XtVaSetValues(arg->name, XtNstring, cp,
 		      XtNinsertPosition, strlen(cp),
 		      NULL);
     } else if (!built) {
@@ -1048,5 +1164,33 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     arg->w = w;
 
     arg->isPoped = True;
-    XtPopup(GetShell(arg->text), XtGrabNone);
+    arg->oldwidth = 0;   
+    shell = GetShell(arg->name);
+
+    XtPopup(shell, XtGrabNone);
+    XtUnmanageChild(arg->cwd_w);
+    XtUnmanageChild(arg->title_w); 
+    XtUnmanageChild(arg->name);
+    XtUnmanageChild(arg->info);
+    XtUnmanageChild(arg->vport); 
+    XtUnmanageChild(arg->ok);    
+    XtUnmanageChild(arg->cancel); 
+    XtUnmanageChild(arg->parent);
+    XtUnmanageChild(arg->browser);
+    for (i=0; i<arg->numf; i++) XtUnmanageChild(arg->format[i]);
+
+    XMapWindow(XtDisplay(arg->parent), XtWindow(arg->parent));
+    XMapWindow(XtDisplay(arg->browser), XtWindow(arg->browser));   
+    XMapWindow(XtDisplay(arg->title_w), XtWindow(arg->title_w));
+    XMapWindow(XtDisplay(arg->info), XtWindow(arg->info));
+    XMapWindow(XtDisplay(arg->vport), XtWindow(arg->vport));
+    XMapWindow(XtDisplay(arg->cwd_w), XtWindow(arg->cwd_w));
+    XMapWindow(XtDisplay(arg->name), XtWindow(arg->name));
+    XMapWindow(XtDisplay(arg->ok), XtWindow(arg->ok));
+    XMapWindow(XtDisplay(arg->cancel), XtWindow(arg->cancel));
+    for (i=0; i<arg->numf; i++)
+        XMapWindow(XtDisplay(arg->format[i]), XtWindow(arg->format[i]));
+
+    XtAddEventHandler(XtParent(arg->parent), StructureNotifyMask, False,
+		      (XtEventHandler) browserResized, (XtPointer) arg);
 }
