@@ -12,22 +12,30 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: lineOp.c,v 1.3 1996/04/19 09:09:01 torsten Exp $ */
+/* $Id: lineOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <math.h>
+#include <stdlib.h>
 #include "xpaint.h"
 #include "misc.h"
 #include "Paint.h"
 #include "ops.h"
 
-static Boolean isRay = False;
-static Boolean isArrow = False;
+static Boolean multiLine = False;
+static Boolean withArrow = False;
+static int headType = 1;
+static int headSize = 15;
+static double headAngle = 25.0;
 
 typedef struct {
     int type;
-    Boolean didUndo, didSet, tracking;
+    Boolean didUndo, didSet, headArrow, tracking;
     int rx, ry;
     int startX, startY, endX, endY;
     int drawn;
@@ -36,13 +44,13 @@ typedef struct {
 } LocalInfo;
 
 static void 
-DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY)
+DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY,
+           Boolean headArrow)
 {
-  XPoint xpoints[3];
+  XPoint xpoints[4];
   Display *dpy; 
   int lw;
-
-  int l, m, ux, uy, lx=0, ly=0, mx=0, my=0;
+  int i, l, m, ux, uy, lx=0, ly=0, mx=0, my=0;
   int DeltaX, DeltaY;
   float norm;
 
@@ -50,18 +58,16 @@ DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY)
   XtVaGetValues(w, XtNlineWidth, &lw, NULL);
   ux = uy = 0;
 
-  if (isArrow) {
+  if (withArrow || headArrow) {
     DeltaX = enX - stX;
     DeltaY = enY - stY;
     norm = sqrt(pow(DeltaX, 2) + pow(DeltaY, 2)) + 0.001;
    
-    if ( lw <= 2 ) {
-       l = 9;
-       m = 5;
-    } else {
-       l = 4*lw+1;
-       m = 2*l/5;
-    }
+    if ( lw <= 2 || headArrow)
+       l = headSize;
+    else
+       l = 4*(lw-1)+headSize;
+    m = l * tan(headAngle*M_PI/180.0);
    
     lx = (int) (0.49 + (float) (abs(DeltaX) * l) / norm);
     ly = (int) (0.49 + (float) (abs(DeltaY) * l) / norm);
@@ -81,19 +87,55 @@ DrawArrow (Widget w, Drawable d, GC gc, int stX,int stY, int enX, int enY)
        ux = 9*lx/10;
        uy = 9*ly/10;
     }
+    if (headType >= 2) {
+       ux /= headType;
+       uy /= headType;
+    }
   }
    
-  XDrawLine(dpy, d, gc, stX, stY, enX-ux, enY-uy);
+  if (!headArrow)
+    XDrawLine(dpy, d, gc, stX, stY, enX-ux, enY-uy);
 
-  if (isArrow) {
-    xpoints[0].x = enX;
-    xpoints[0].y = enY;
-    xpoints[1].x = enX - lx + mx;
-    xpoints[1].y = enY - ly + my;
-    xpoints[2].x = enX - lx - mx;
-    xpoints[2].y = enY - ly - my;
-   
-    XFillPolygon(dpy, d, gc, xpoints, 3, Complex, CoordModeOrigin);
+  if (withArrow || headArrow) {
+    if (headArrow) {
+       xpoints[0].x = stX;
+       xpoints[0].y = stY;
+    } else {
+       xpoints[0].x = enX;
+       xpoints[0].y = enY;
+    }
+    if (headType == 1) {
+       xpoints[1].x = xpoints[0].x - lx + mx;
+       xpoints[1].y = xpoints[0].y - ly + my;
+       xpoints[2].x = xpoints[0].x - lx - mx;
+       xpoints[2].y = xpoints[0].y - ly - my;
+       XFillPolygon(dpy, d, gc, xpoints, 3, Complex, CoordModeOrigin);
+    }
+    if (headType == 2) {
+       xpoints[1].x = xpoints[0].x - lx + mx;
+       xpoints[1].y = xpoints[0].y - ly + my;
+       xpoints[2].x = xpoints[0].x - lx/2;
+       xpoints[2].y = xpoints[0].y - ly/2;
+       xpoints[3].x = xpoints[0].x - lx - mx;
+       xpoints[3].y = xpoints[0].y - ly - my;
+       XFillPolygon(dpy, d, gc, xpoints, 4, Complex, CoordModeOrigin);
+    }
+    if (headType == 3) {
+       xpoints[1].x = xpoints[0].x - lx + mx;
+       xpoints[1].y = xpoints[0].y - ly + my;
+       xpoints[2].x = xpoints[0].x - lx/3;
+       xpoints[2].y = xpoints[0].y - ly/3;
+       xpoints[3].x = xpoints[0].x - lx - mx;
+       xpoints[3].y = xpoints[0].y - ly - my;
+       XFillPolygon(dpy, d, gc, xpoints, 4, Complex, CoordModeOrigin);
+       lx /= 3;
+       ly /= 3;
+       for (i=0; i<4; i++) {
+           xpoints[i].x -= lx;
+           xpoints[i].y -= ly;
+       }
+       XFillPolygon(dpy, d, gc, xpoints, 4, Complex, CoordModeOrigin);
+    }
   }
 }
  
@@ -106,7 +148,9 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     if ((event->state & AllButtonsMask) != 0)
 	return;
 
-    if (event->button == Button1 && ((!l->tracking && isRay) || !isRay)) {
+    if (event->button >= Button4) return;
+   
+    if (event->button == Button1 && ((!l->tracking && multiLine) || !multiLine)) {
 	l->rx = info->x;
 	l->ry = info->y;
 	l->startX = event->x;
@@ -120,7 +164,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     } else if (event->button == Button2) {
 	if (l->drawn)
 	    DrawArrow(w, info->drawable, l->gcx,
-		      l->startX, l->startY, l->endX, l->endY);
+		      l->startX, l->startY, l->endX, l->endY, l->headArrow);
 	l->tracking = False;
 	l->drawn = False;
         return;
@@ -128,13 +172,13 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     if (l->tracking) {
 	if (l->drawn)
 	    DrawArrow(w, info->drawable, l->gcx,
-		      l->startX, l->startY, l->endX, l->endY);
+		      l->startX, l->startY, l->endX, l->endY, l->headArrow);
 
 	l->endX = event->x;
 	l->endY = event->y;
 
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
 	l->drawn = True;
     }
 }
@@ -142,12 +186,13 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 static void 
 motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
+
     if (!l->tracking)
 	return;
 
     if (l->drawn)
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
 
     l->type = (event->state & ShiftMask);
 
@@ -188,7 +233,7 @@ motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
      */
     if ((l->drawn = (l->startX != l->endX || l->startY != l->endY)))
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
 }
 
 static void 
@@ -227,7 +272,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     if (l->drawn && info->surface == opWindow) {
 	DrawArrow(w, info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
 	l->drawn = False;
     }
 
@@ -272,7 +317,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     SetCapAndJoin(w, info->first_gc, JoinMiter);
     DrawArrow(w, info->drawable, info->first_gc,
-	      l->rx, l->ry, l->endX, l->endY);
+	      l->rx, l->ry, l->endX, l->endY, l->headArrow);
 
     if (info->surface == opPixmap) {
 	XYtoRECT(l->rx, l->ry, l->endX, l->endY, &undo);
@@ -287,7 +332,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	PwUpdate(w, &undo, False);
     }
 
-    if (!isRay)
+    if (!multiLine || l->headArrow)
         l->tracking = False;
 }
 
@@ -301,6 +346,8 @@ LineAdd(Widget w)
 
     l->tracking = False;
     l->gcx = GetGCX(w);
+    l->headArrow = False;
+
     XtVaSetValues(w, XtNcompress, True, NULL);
 
     OpAddEventHandler(w, opWindow, ButtonPressMask, FALSE,
@@ -325,35 +372,75 @@ LineRemove(Widget w, void *p)
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
 
-    if (isRay && l->tracking && l->drawn)
+    if (multiLine && l->tracking && l->drawn)
 	DrawArrow(w, l->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
+
+    XtFree((XtPointer) l);
+}
+
+void *
+ArrowAdd(Widget w)
+{
+    LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
+
+    l->tracking = False;
+    l->gcx = GetGCX(w);
+    l->headArrow = True;
+
+    XtVaSetValues(w, XtNcompress, True, NULL);
+
+    OpAddEventHandler(w, opWindow, ButtonPressMask, FALSE,
+		      (OpEventProc) press, l);
+    OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
+		      (OpEventProc) motion, l);
+    OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
+		      (OpEventProc) release, l);
+    SetCrossHairCursor(w);
+    return l;
+}
+
+void
+ArrowRemove(Widget w, void *p)
+{
+    LocalInfo *l = (LocalInfo *) p;
+
+    OpRemoveEventHandler(w, opWindow, ButtonPressMask, FALSE,
+			 (OpEventProc) press, l);
+    OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
+			 (OpEventProc) motion, l);
+    OpRemoveEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
+			 (OpEventProc) release, l);
+
+    if (multiLine && l->tracking && l->drawn)
+	DrawArrow(w, l->drawable, l->gcx,
+		  l->startX, l->startY, l->endX, l->endY, l->headArrow);
 
     XtFree((XtPointer) l);
 }
 
 Boolean
-RayGetStyle(void)
+MultiGetStyle(void)
 {
-    return isRay;
+    return multiLine;
 }
 
 void 
-RaySetStyle(Boolean flag)
+MultiSetStyle(Boolean flag)
 {
-    isRay = flag;
+    multiLine = flag;
 }
 
 Boolean
 ArrowGetStyle(void)
 {
-    return isArrow;
+    return withArrow;
 }
 
 void 
 ArrowSetStyle(Boolean flag)
 {
-    isArrow = flag;
+    withArrow = flag;
 }
 
 void
@@ -400,3 +487,13 @@ DashSetStyle(char *dashstyle)
         Global.dashnumber = 0;
 }
 
+/*
+**  Those public functions
+*/
+void
+ArrowHeadSetParameters(int t, int s, float a)
+{
+    headType = t;
+    headSize = s;
+    headAngle = a;
+}
