@@ -13,7 +13,13 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: main.c,v 1.18 2000/09/11 18:42:58 torsten Exp $ */
+/* $Id: main.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtScreen XTSCREEN
+#define XtWindow XTWINDOW
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
@@ -37,6 +43,7 @@
 #include "graphic.h"
 #include "messages.h"
 #include "protocol.h"
+#include "image.h"
 #include "rw/rwTable.h"
 
 #include "XPaintIcon.xpm"
@@ -47,6 +54,7 @@ String *msgText;
 int msgNum;
 
 char lang[10];
+char *routine;
 
 extern void BrushInit(Widget toplevel);
 
@@ -70,6 +78,7 @@ typedef struct {
     Boolean nomenubar;
     Boolean popped;
     Boolean snapshot;
+    Boolean clipboard;
     Boolean horizontal;
     Boolean nowarn;
 } AppInfo;
@@ -104,6 +113,8 @@ static XtResource resources[] =
      XtOffset(AppInfo *, popped), XtRImmediate, (XtPointer) False},
     {"snapshot", "Snapshot", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, snapshot), XtRImmediate, (XtPointer) False},
+    {"clipboard", "Clipboard", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, clipboard), XtRImmediate, (XtPointer) False},
     {"horizontal", "Horizontal", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, horizontal), XtRImmediate, (XtPointer) False},
     {"nowarn", "NoWarn", XtRBoolean, sizeof(Boolean),
@@ -126,6 +137,7 @@ static XrmOptionDescRec options[] =
     {"-nomenubar", ".nomenubar", XrmoptionNoArg, (XtPointer) "True"},
     {"-popped", ".popped", XrmoptionNoArg, (XtPointer) "True"},
     {"-snapshot", ".snapshot", XrmoptionNoArg, (XtPointer) "True"},
+    {"-clipboard", ".clipboard", XrmoptionNoArg, (XtPointer) "True"},
     {"-horizontal", ".horizontal", XrmoptionNoArg, (XtPointer) "True"},
     {"-nowarn", ".nowarn", XrmoptionNoArg, (XtPointer) "True"},
     {"-help", ".help", XrmoptionNoArg, (XtPointer) "command"},
@@ -724,6 +736,7 @@ main(int argc, char *argv[])
     Global.dashnumber = 0;
     Global.dashoffset = 0;
     Global.numregions = 0;
+    Global.transparent = 0;
     Global.regiondata = NULL;
 
     /*
@@ -817,20 +830,35 @@ main(int argc, char *argv[])
     **  Now open any file on the command line
      */
     if (argc != 1) {
-	LocalInfo *l = XtNew(LocalInfo);
-	l->pos = 0;
-	l->toplevel = toplevel;
-	l->nfiles = argc - 1;
-	l->files = (char **) XtCalloc(argc, sizeof(char *));
-	for (i = 1; i < argc; i++)
-	    l->files[i - 1] = argv[i];
-	l->id = XtAppAddWorkProc(app, (XtWorkProc) workProc, (XtPointer) l);
+        if (appInfo.clipboard) {
+	   Image * image;
+	   for (i = 1; i < argc; i++) {
+  	       image = ImageFromFile(argv[i]);
+               if (image)
+                  ImageToMemory(image);
+	   }
+	} else {
+  	   LocalInfo *l = XtNew(LocalInfo);
+	   l->pos = 0;
+	   l->toplevel = toplevel;
+  	   l->nfiles = argc - 1;
+	   l->files = (char **) XtCalloc(argc, sizeof(char *));
+           for (i = 1; i < argc; i++)
+	       l->files[i - 1] = argv[i];
+	   l->id = XtAppAddWorkProc(app, (XtWorkProc) workProc, (XtPointer) l);
+	}
     } else if (appInfo.popped) {
 	/*
 	**  If nothing is coming up, bring up a blank canvas
 	 */
 	GraphicCreate(toplevel, 0);
     }
+
+    XtUnmanageChild(Global.bar);
+    XMapWindow(dpy, XtWindow(Global.bar));
+    XtUnmanageChild(Global.back);
+    XMapWindow(dpy, XtWindow(Global.back));
+
     /*
     **  MainLoop
      */
