@@ -10,7 +10,10 @@
 ** implied warranty.
  */
 
-/* $Id: libpnmrw.c,v 1.10 2000/10/19 20:56:14 torsten Exp $ */
+/* $Id: libpnmrw.c,v 1.22 2005/03/20 20:15:34 demailly Exp $ */
+
+#include <stdlib.h>
+#include <string.h>
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
@@ -47,12 +50,7 @@ typedef char *String;
 #include <strings.h>
 #endif				/*SYSV */
 
-void *malloc(int);
-void free(void *);
-#if defined(__EMX__) || defined(__CYGWIN__)
 #include <errno.h>
-#endif
-
 
 /* Definitions. */
 
@@ -142,17 +140,20 @@ pm_perror(char *reason)
 #ifdef __CYGWIN__
 #  define sys_errlist _sys_errlist
 #else
+#ifdef MISSING_STRERROR
     extern char *sys_errlist[];
 #endif
 #endif
 #endif
 #endif
-#if !defined(__EMX__) || defined(__CYGWIN__)
-    extern int errno;
 #endif
     char *e;
 
+#ifdef MISSING_STRERROR
     e = (char *)sys_errlist[errno];
+#else
+    e = (char *)strerror(errno);   
+#endif
 
     if (reason != 0 && reason[0] != '\0')
 	fprintf(stderr, "%s: %s - %s\n", progname, reason, e);
@@ -490,7 +491,7 @@ pgm_readpgmrow(FILE * file, gray * grayrow, int cols, gray maxval, int format)
 static void
 pgm_writepgminit(FILE * file, int cols, int rows, gray maxval, int forceplain)
 {
-    if (maxval <= 255 && !forceplain)
+    if (!forceplain)
 	fprintf(
 		   file, "%c%c\n%d %d\n%d\n", PGM_MAGIC1, RPGM_MAGIC2,
 		   cols, rows, maxval);
@@ -544,7 +545,7 @@ pgm_writepgmrowplain(FILE * file, gray * grayrow, int cols, gray maxval)
 static int
 pgm_writepgmrow(FILE * file, gray * grayrow, int cols, gray maxval, int forceplain)
 {
-    if (maxval <= 255 && !forceplain)
+    if (!forceplain)
 	return pgm_writepgmrowraw(file, grayrow, cols, maxval);
     else
 	return pgm_writepgmrowplain(file, grayrow, cols, maxval);
@@ -622,7 +623,7 @@ ppm_readppmrow(FILE * file, pixel * pixelrow, int cols,
 static void
 ppm_writeppminit(FILE * file, int cols, int rows, pixval maxval, int forceplain)
 {
-    if (maxval <= 255 && !forceplain)
+    if (!forceplain)
 	fprintf(
 		   file, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,
 		   cols, rows, maxval);
@@ -692,7 +693,7 @@ static int
 ppm_writeppmrow(FILE * file, pixel * pixelrow, int cols,
 		pixval maxval, int forceplain)
 {
-    if (maxval <= 255 && !forceplain)
+    if (!forceplain)
 	return ppm_writeppmrowraw(file, pixelrow, cols, maxval);
     else
 	return ppm_writeppmrowplain(file, pixelrow, cols, maxval);
