@@ -12,7 +12,7 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: rwTable.c,v 1.9 2000/10/19 20:56:23 torsten Exp $ */
+/* $Id: rwTable.c,v 1.22 2005/03/20 20:15:34 demailly Exp $ */
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
@@ -54,6 +54,7 @@ static char RWtableMsg[512];
 **	the "variable" name version of these. ]
  */
 
+extern Image *ReadBMP(char *);
 extern Image *ReadXBM(char *);
 extern Image *ReadXWD(char *);
 extern Image *ReadPNM(char *);
@@ -66,6 +67,7 @@ extern Image *ReadJPEG(char *);
 extern Image *ReadICO(char *);
 extern Image *ReadScriptC(char *);
 extern Image *ReadPS(char *);
+extern int WriteBMP(char *, Image *);
 extern int WriteXBM(char *, Image *);
 extern int WriteXWD(char *, Image *);
 extern int WritePNM(char *, Image *);
@@ -79,6 +81,8 @@ extern int WriteJPEG(char *, Image *);
 extern int WriteICO(char *, Image *);
 extern int WritePDF(char *, Image *);
 extern int WritePS(char *, Image *);
+extern int TestBMP(char *);
+extern int TestICO(char *);
 extern int TestPNM(char *);
 extern int TestXWD(char *);
 extern int TestXBM(char *);
@@ -88,7 +92,6 @@ extern int TestGIF(char *);
 extern int TestSGI(char *);
 extern int TestTIFF(char *);
 extern int TestJPEG(char *);
-extern int TestICO(char *);
 extern int TestScriptC(char *);
 extern int TestPS(char *);
 extern int TestPDF(char *);
@@ -100,23 +103,24 @@ extern int TestPDF(char *);
 static ImageTypes RWtable[] =
 {
     {"Auto_Detect",      readMagic, NULL,      NULL    },
-    {"GIF_Format",       ReadGIF,   WriteGIF,  TestGIF },
-    {"ICO_Format",       ReadICO,   WriteICO,  TestICO },
-#ifdef HAVE_JPEG
-    {"JPEG_Format",      ReadJPEG,  WriteJPEG, TestJPEG},
-#endif
 #ifdef HAVE_PNG   /* ReadPNG does all PNG files; no need for two entries */
     {"PNG_Format",       ReadPNG,   WritePNGn, TestPNG },
     {"PNG_Interlaced",	 NULL,      WritePNGi, TestPNG },
 #endif
+    {"GIF_Format",       ReadGIF,   WriteGIF,  TestGIF },
+#ifdef HAVE_JPEG
+    {"JPEG_Format",      ReadJPEG,  WriteJPEG, TestJPEG},
+#endif
+#ifdef HAVE_TIFF
+    {"TIFF_Format",      ReadTIFF,  WriteTIFF, TestTIFF},
+#endif
+    {"BMP_Format",       ReadBMP,   WriteBMP,  TestBMP },
+    {"ICO_Format",       ReadICO,   WriteICO,  TestICO },
     {"PPM_Format",       ReadPNM,   WritePNM,  TestPNM },
     {"PS_Format",        ReadPS,    WritePS,   TestPS  },
     {"PDF_Format",       ReadPS,    WritePDF,  TestPDF },
 #ifdef HAVE_SGI
     {"SGI_Format",       ReadSGI,   WriteSGI,  TestSGI },
-#endif
-#ifdef HAVE_TIFF
-    {"TIFF_Format",      ReadTIFF,  WriteTIFF, TestTIFF},
 #endif
     {"XBM_Format",       ReadXBM,   WriteXBM,  TestXBM },
     {"XPM_Format",       ReadXPM,   WriteXPM,  TestXPM },
@@ -137,7 +141,6 @@ static char *usedMagicReader = NULL;
 static Image *
 readMagic(char *file)
 {
-    extern int errno;
     int i;
 
     errno = 0;
@@ -259,7 +262,6 @@ RWGetMsg()
 #endif
 #endif
 #endif
-    extern int errno;
 
     if (RWtableMsg[0] == '\0') {
 	if (errno == 0)
