@@ -15,19 +15,22 @@
 /* Polygon selection code and other portions */
 /* Copyright 1995, 1996 Torsten Martinsen    */
 
-/* $Id: selectOp.c,v 1.7 1997/07/14 17:37:11 torsten Exp $ */
+/* $Id: selectOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdio.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Scrollbar.h>
-#include <X11/Xaw/Command.h>
 #include <X11/Shell.h>
 #include <X11/keysym.h>
-#include <stdio.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/Command.h"
+
 #include "xpaint.h"
 #include "Paint.h"
 #include "PaintP.h"
@@ -37,6 +40,7 @@
 #include "misc.h"
 #include "operation.h"
 #include "ops.h"
+#include "region.h"
 
 /*
 **  It is now historical, that the Box and Area selection
@@ -48,7 +52,7 @@ static int shapeMode = 0;
 
 typedef struct {
     int areaType;		/* 0: box/ellipse, 1: lasso */
-    Boolean lastType;		/* True: square/circle */
+    int lastType;		/* square/circle */
     Boolean tracking, first;
 
     Widget w;
@@ -64,11 +68,6 @@ typedef struct {
     int size, npoint;
     XPoint *real_p, *window_p;
 
-    /*
-     * Additional info for polygon
-     */
-    int startX, endX, startY, endY, button;
-
     /*
     **  The rectangles for a box (also used for ellipse)
      */
@@ -85,21 +84,30 @@ typedef struct {
     XRectangle pixBox;
 } LocalInfo;
 
-static Boolean chromaCut(Widget w, LocalInfo * l, Pixmap * mask);
-
-#define MKRECT(rect, sx, sy, ex, ey, typeFlag) do {			\
-	if (typeFlag) {		/* square */				\
-		(rect)->width  = MIN(ABS(sx - ex),ABS(sy - ey));	\
+#define MKRECT(rect, sx, sy, ex, ey, typeFlag) {			\
+	if (typeFlag == 1) {	/* square */				\
+		(rect)->width  = MAX(ABS(sx - ex),ABS(sy - ey));	\
 		(rect)->height = (rect)->width;				\
 		(rect)->x = (ex - sx < 0) ? sx - (rect)->width : sx;	\
 		(rect)->y = (ey - sy < 0) ? sy - (rect)->height : sy;	\
+	} else if (typeFlag == 2) {	/* center */			\
+		(rect)->x      = sx - ABS(sx - ex);			\
+		(rect)->y      = sy - ABS(sy - ey);			\
+		(rect)->width  = 2*ABS(sx - ex);			\
+		(rect)->height = 2*ABS(sy - ey);			\
+	} else if (typeFlag == 3) {	/* center, square */		\
+		(rect)->width  = MAX(ABS(sx - ex),ABS(sy - ey));	\
+		(rect)->x      = sx - (rect)->width;			\
+		(rect)->y      = sy - (rect)->width;			\
+		(rect)->width  *= 2;					\
+		(rect)->height = (rect)->width;				\
 	} else {		/* no constraints */			\
 		(rect)->x      = MIN(sx, ex);				\
 		(rect)->y      = MIN(sy, ey);				\
 		(rect)->width  = MAX(sx, ex) - (rect)->x;		\
 		(rect)->height = MAX(sy, ey) - (rect)->y;		\
 	}								\
-} while (0)
+}
 
 /* 
  * If a region is already visible, unselect everything !!
@@ -116,8 +124,8 @@ checkVisibility(Widget w, LocalInfo * l, XButtonEvent * event)
         l->tracking = False;
 	l->first = False;
 	l->npoint = 0;
-	l->endX = l->startX = l->lastX = l->wx = event->x;
-	l->endY = l->startY = l->lastY = l->wy = event->y;
+	l->lastX = l->wx = event->x;
+	l->lastY = l->wy = event->y;
 	return 1;
     } else
         return 0;
@@ -133,6 +141,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     **  Check to make sure all buttons are up, before doing this
      */
 
+    if (event->button >= Button4) return;    
     if ((event->state & AllButtonsMask) != 0)
 	return;
     if (event->button == Button3) return;
@@ -165,8 +174,8 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     l->tracking = True;
 }
 
-static void 
-keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
+void 
+selectKeyPress(Widget w, void * l, XKeyEvent * event, void * info)
 {
     KeySym keysym;
     int len, i, dx, dy;
@@ -188,11 +197,11 @@ keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
     case XK_Right:
 	dx = 1;
 	break;
-    case XK_Alt_L:
-    case XK_Alt_R:
+    case XK_space:
+        Global.transparent = !Global.transparent;
         PwRegionTear(w);
-	RegionTransparency((PaintWidget) w, 0);
-	return;
+	RegionTransparency((PaintWidget) w);
+	break;
     default:
 	break;
     }
@@ -215,121 +224,16 @@ keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
     }
 }
 
-static void 
-keyRelease(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
+void 
+selectKeyRelease(Widget w, void * l, XKeyEvent * event, void * info)
 {
     KeySym keysym;
-    int len, i, dx, dy;
+    int len;
     char buf[21];
 
     len = XLookupString(event, buf, sizeof(buf) - 1, &keysym, NULL);
 
-    if (keysym==XK_Alt_L || keysym==XK_Alt_R)
-       RegionTransparency((PaintWidget) w, 1);
-}
-
-static void 
-finishPolyBand(Widget w, LocalInfo * l, OpInfo * info)
-{
-    int xmin, xmax, ymin, ymax, i, width, height;
-    Pixmap mask;
-    GC gc;
-    XPoint tmp;
-
-    if (!l->tracking)
-	return;
-
-    l->tracking = False;
-
-    if (l->drawn)
-	XDrawLines(XtDisplay(w), XtWindow(w), l->gcx,
-		   l->window_p, l->npoint, CoordModeOrigin);
-    PwUpdateDrawable(w, info->drawable, NULL);
-    if (l->npoint <= 2)
-	return;
-    l->window_p[l->npoint] = l->window_p[0];
-    l->real_p[l->npoint] = l->real_p[0];
-    l->npoint++;
-
-    xmin = xmax = l->real_p[0].x;
-    ymin = ymax = l->real_p[0].y;
-    for (i = l->npoint - 1; i > 0; i--) {
-	xmin = MIN(xmin, l->real_p[i].x);
-	ymin = MIN(ymin, l->real_p[i].y);
-	xmax = MAX(xmax, l->real_p[i].x);
-	ymax = MAX(ymax, l->real_p[i].y);
-
-	l->window_p[i].x = l->window_p[i].x - l->window_p[i - 1].x;
-	l->window_p[i].y = l->window_p[i].y - l->window_p[i - 1].y;
-	l->real_p[i].x = l->real_p[i].x - l->real_p[i - 1].x;
-	l->real_p[i].y = l->real_p[i].y - l->real_p[i - 1].y;
-    }
-
-    XYtoRECT(xmin, ymin, xmax, ymax, &l->pixBox);
-    XtVaGetValues(w, XtNdrawWidth, &width, XtNdrawHeight, &height, NULL);
-    if (l->pixBox.x + l->pixBox.width > width)
-	l->pixBox.width = width - l->pixBox.x;
-    if (l->pixBox.y + l->pixBox.height > height)
-	l->pixBox.height = height - l->pixBox.y;
-    if (l->pixBox.x < 0) {
-	l->pixBox.width += l->pixBox.x;
-	l->pixBox.x = 0;
-    }
-    if (l->pixBox.y < 0) {
-	l->pixBox.height += l->pixBox.y;
-	l->pixBox.y = 0;
-    }
-    mask = XCreatePixmap(XtDisplay(w), XtWindow(w),
-			 l->pixBox.width, l->pixBox.height, 1);
-    gc = XCreateGC(XtDisplay(w), mask, 0, 0);
-    XSetFunction(XtDisplay(w), gc, GXclear);
-    XFillRectangle(XtDisplay(w), mask, gc, 0, 0,
-		   l->pixBox.width, l->pixBox.height);
-    XSetFunction(XtDisplay(w), gc, GXset);
-    tmp = l->real_p[0];
-    l->real_p[0].x = l->real_p[0].x - xmin;
-    l->real_p[0].y = l->real_p[0].y - ymin;
-    XFillPolygon(XtDisplay(w), mask, gc, l->real_p, l->npoint,
-		 Complex, CoordModePrevious);
-    l->real_p[0] = tmp;
-
-    XFreeGC(XtDisplay(w), gc);
-
-    if (l->pixBox.width <= 1 || l->pixBox.height <= 1) {
-	PwRegionFinish(w, True);
-	return;
-    }
-    if (cutMode != 0 && !chromaCut(w, l, &mask)) {
-	PwRegionFinish(w, True);
-	return;
-    }
-    PwRegionSet(w, &l->pixBox, None, mask);
-}
-
-static void 
-pressPolyBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
-{
-    if (l->first) {
-        if (checkVisibility(w, l, event)) return;
-    }
-    if (!l->tracking && (event->button == Button1)) {
-	l->endX = l->startX = event->x;
-	l->endY = l->startY = event->y;
-
-	l->button = event->button;
-
-	l->window_p[0].x = event->x;
-	l->window_p[0].y = event->y;
-	l->real_p[0].x = info->x;
-	l->real_p[0].y = info->y;
-	l->npoint = 1;
-
-	l->drawn = False;
-	l->tracking = True;
-	l->first = True;
-    } else if (l->tracking && event->button == Button2) {
-	finishPolyBand(w, l, info);
-    }
+    /* Noop at this time */
 }
 
 /*
@@ -347,21 +251,24 @@ motionBoxBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 	MKRECT(&rect, l->window_p[0].x, l->window_p[0].y,
 	       l->lastX, l->lastY, l->lastType);
 	if (shapeMode == 0)
-	    XDrawRectangles(XtDisplay(w), XtWindow(w), l->gcx, &rect, 1);
+	    DrawBox(XtDisplay(w), XtWindow(w), l->gcx, &rect, 0);
 	else
 	    XDrawArc(XtDisplay(w), XtWindow(w), l->gcx, rect.x, rect.y,
 		     rect.width, rect.height, 0, 360 * 64);
     }
     l->lastX = event->x - info->zoom / 2;
     l->lastY = event->y - info->zoom / 2;
-    l->lastType = (event->state & ShiftMask);
+    if (shapeMode==0)
+        l->lastType = (1&BoxGetStyle()?2:0) + ((event->state & ShiftMask)?1:0);
+    if (shapeMode==1)
+        l->lastType = (CircleGetStyle()?2:0) +((event->state & ShiftMask)?1:0);
 
     if ((l->drawn =
        (l->window_p[0].x != l->lastX && l->window_p[0].y != l->lastY))) {
 	MKRECT(&rect, l->window_p[0].x, l->window_p[0].y,
 	       l->lastX, l->lastY, l->lastType);
 	if (shapeMode == 0)
-	    XDrawRectangles(XtDisplay(w), XtWindow(w), l->gcx, &rect, 1);
+	    DrawBox(XtDisplay(w), XtWindow(w), l->gcx, &rect, 0);
 	else
 	    XDrawArc(XtDisplay(w), XtWindow(w), l->gcx, rect.x, rect.y,
 		     rect.width, rect.height, 0, 360 * 64);
@@ -369,7 +276,7 @@ motionBoxBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 }
 
 static void 
-motionLassoBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
+motionSelectFreehandBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
 
     if (l->npoint == -1 || !l->tracking)  
@@ -405,25 +312,6 @@ motionLassoBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
     }
 }
 
-static void 
-motionPolyBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
-{
-
-    if (l->npoint == -1 || !l->tracking)
-        return;
-
-    if (l->drawn)
-	XDrawLine(XtDisplay(w), info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
-
-    l->endX = event->x;
-    l->endY = event->y;
-
-    if ((l->drawn = ((l->startX != l->endX) || (l->startY != l->endY))))
-	XDrawLine(XtDisplay(w), info->drawable, l->gcx,
-		  l->startX, l->startY, l->endX, l->endY);
-}
-
 static void 
 motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
@@ -437,11 +325,11 @@ motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
     if (l->areaType == 0)
 	motionBoxBand(w, l, event, info);
     else
-	motionLassoBand(w, l, event, info);
+	motionSelectFreehandBand(w, l, event, info);
 }
 
-static Boolean
-chromaCut(Widget w, LocalInfo * l, Pixmap * mask)
+Boolean
+chromaCut(Widget w, XRectangle * r, Pixmap * mask)
 {
     GC gc;
     XImage *src, *mimg;
@@ -457,8 +345,8 @@ chromaCut(Widget w, LocalInfo * l, Pixmap * mask)
     Colormap cmap;
     int step;
 
-    if (l)
-        rect = PwScaleRectangle(w, &l->pixBox);
+    if (r)
+        rect = PwScaleRectangle(w, r);
     else {
         rect = (XRectangle *) xmalloc(sizeof(XRectangle));
 	rect->x = 0;
@@ -545,7 +433,7 @@ releaseBoxBand(Widget w, LocalInfo * l, XButtonEvent * event,
 	MKRECT(&rect, l->window_p[0].x, l->window_p[0].y,
 	       l->lastX, l->lastY, l->lastType);
 	if (shapeMode == 0)
-	    XDrawRectangles(XtDisplay(w), XtWindow(w), l->gcx, &rect, 1);
+	    DrawBox(XtDisplay(w), XtWindow(w), l->gcx, &rect, 0);
 	else
 	    XDrawArc(XtDisplay(w), XtWindow(w), l->gcx, rect.x, rect.y,
 		     rect.width, rect.height, 0, 360 * 64);
@@ -566,7 +454,7 @@ releaseBoxBand(Widget w, LocalInfo * l, XButtonEvent * event,
 	l->pixBox.height += l->pixBox.y;
 	l->pixBox.y = 0;
     }
-    if (shapeMode == 0)
+    if (shapeMode == 0 && (BoxGetStyle()&2) == 0)
 	return;
 
     /* to avoid crash when selecting a single point as ellipse - ACZ */
@@ -580,14 +468,21 @@ releaseBoxBand(Widget w, LocalInfo * l, XButtonEvent * event,
     XFillRectangle(XtDisplay(w), *mask, gc, 0, 0,
 		   l->pixBox.width, l->pixBox.height);
     XSetFunction(XtDisplay(w), gc, GXset);
-    XFillArc(XtDisplay(w), *mask, gc, 0, 0,
-	     l->pixBox.width, l->pixBox.height, 0, 360 * 64);
+    if (shapeMode == 0) {
+        rect.x = 0;
+        rect.y = 0;
+        rect.width = l->pixBox.width;
+        rect.height = l->pixBox.height;
+        DrawBox(XtDisplay(w), *mask, gc, &rect, 1);
+    } else
+        XFillArc(XtDisplay(w), *mask, gc, 0, 0,
+	         l->pixBox.width, l->pixBox.height, 0, 360 * 64);
 
     XFreeGC(XtDisplay(w), gc);
 }
 
 static Boolean
-releaseLassoBand(Widget w, LocalInfo * l, XButtonEvent * event,
+releaseSelectFreehandBand(Widget w, LocalInfo * l, XButtonEvent * event,
 		 OpInfo * info, Pixmap * mask)
 {
     int xmax, xmin, ymax, ymin;
@@ -655,35 +550,6 @@ releaseLassoBand(Widget w, LocalInfo * l, XButtonEvent * event,
     return True;
 }
 
-static void 
-releasePolyBand(Widget w, LocalInfo * l, XButtonEvent * event,
-		OpInfo * info)
-{
-    if (l->first) {       	
-	l->first = False;
-	return;
-    } else {
-	l->endX = event->x;
-	l->endY = event->y;
-	l->startX = event->x;
-	l->startY = event->y;
-    }
-
-    if (l->npoint > l->size - 3) {
-	l->size += 32;
-	l->real_p = (XPoint *) XtRealloc((XtPointer) l->real_p,
-					 l->size * sizeof(XPoint));
-	l->window_p = (XPoint *) XtRealloc((XtPointer) l->window_p,
-					   l->size * sizeof(XPoint));
-    }
-    l->window_p[l->npoint].x = event->x;
-    l->window_p[l->npoint].y = event->y;
-    l->real_p[l->npoint].x = info->x;
-    l->real_p[l->npoint].y = info->y;
-    l->npoint++;
-}
-
-
 static void 
 release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
@@ -693,6 +559,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     /*
     **  Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;      
     mask = AllButtonsMask;
     switch (event->button) {
     case Button1:
@@ -719,7 +586,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 
     if (l->areaType == 0)
 	releaseBoxBand(w, l, event, info, &mpix);
-    else if (releaseLassoBand(w, l, event, info, &mpix) == False) {
+    else if (releaseSelectFreehandBand(w, l, event, info, &mpix) == False) {
 	PwRegionFinish(w, True);
 	return;
     }
@@ -727,7 +594,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	PwRegionFinish(w, True);
 	return;
     }
-    if (cutMode != 0 && !chromaCut(w, l, &mpix)) {
+    if (cutMode != 0 && !chromaCut(w, l?&l->pixBox:NULL, &mpix)) {
 	PwRegionFinish(w, True);
 	return;
     }
@@ -737,13 +604,16 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 /*
 **  Those public functions
  */
-void *
-SelectBoxAdd(Widget w)
+static void *
+SelectBoxOrOvalAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
 
     l->areaType = 0;
-    l->lastType = 0;
+    if (shapeMode==0)
+        l->lastType = (1&BoxGetStyle()? 2:0);
+    if (shapeMode==1)
+        l->lastType = (CircleGetStyle()? 2:0);
     l->npoint = -1;
     l->gcx = GetGCX(w);
 
@@ -760,14 +630,28 @@ SelectBoxAdd(Widget w)
     OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
-		      (OpEventProc) keyPress, l);
+		      (OpEventProc) selectKeyPress, l);
     OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		      (OpEventProc) keyRelease, l);
+		      (OpEventProc) selectKeyRelease, l);
     SetCrossHairCursor(w);
 
     return l;
 }
 
+void *
+SelectBoxAdd(Widget w)
+{
+    shapeMode = 0;
+    return SelectBoxOrOvalAdd(w);
+}
+
+void *
+SelectOvalAdd(Widget w)
+{
+    shapeMode = 1;
+    return SelectBoxOrOvalAdd(w);
+}
+
 void 
 SelectBoxRemove(Widget w, void *p)
 {
@@ -780,9 +664,9 @@ SelectBoxRemove(Widget w, void *p)
     OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
-			 (OpEventProc) keyPress, l);
+			 (OpEventProc) selectKeyPress, l);
     OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-			 (OpEventProc) keyRelease, l);
+			 (OpEventProc) selectKeyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
@@ -790,8 +674,14 @@ SelectBoxRemove(Widget w, void *p)
     XtFree((XtPointer) l);
 }
 
+void 
+SelectOvalRemove(Widget w, void *p)
+{
+    SelectBoxRemove(w, p);
+}
+
 void *
-LassoAdd(Widget w)
+SelectFreehandAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
 
@@ -812,9 +702,9 @@ LassoAdd(Widget w)
     OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
-		      (OpEventProc) keyPress, l);
+		      (OpEventProc) selectKeyPress, l);
     OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		      (OpEventProc) keyRelease, l);
+		      (OpEventProc) selectKeyRelease, l);
 
     SetPencilCursor(w);
 
@@ -822,7 +712,7 @@ LassoAdd(Widget w)
 }
 
 void 
-LassoRemove(Widget w, void *p)
+SelectFreehandRemove(Widget w, void *p)
 {
     LocalInfo *l = (LocalInfo *) p;
 
@@ -833,9 +723,9 @@ LassoRemove(Widget w, void *p)
     OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
-			 (OpEventProc) keyPress, l);
+			 (OpEventProc) selectKeyPress, l);
     OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-			 (OpEventProc) keyRelease, l);
+			 (OpEventProc) selectKeyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
@@ -843,68 +733,6 @@ LassoRemove(Widget w, void *p)
     XtFree((XtPointer) l);
 }
 
-
-void *
-SelectPolyAdd(Widget w)
-{
-    LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
-
-    l->areaType = 2;
-    l->drawn = False;
-    l->first = True;
-    l->tracking = False;
-    l->npoint = -1;
-    l->gcx = GetGCX(w);
-
-    XtVaSetValues(w, XtNcompress, False, NULL);
-
-    l->size = 256;
-    l->real_p = (XPoint *) XtCalloc(sizeof(XPoint), l->size);
-    l->window_p = (XPoint *) XtCalloc(sizeof(XPoint), l->size);
-
-    OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
-		      (OpEventProc) pressPolyBand, l);
-    OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
-		      (OpEventProc) motionPolyBand, l);
-    OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
-		      (OpEventProc) releasePolyBand, l);
-    OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
-		      (OpEventProc) keyPress, l);
-    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-		      (OpEventProc) keyRelease, l);
-    SetPencilCursor(w);
-
-    return l;
-}
-
-void 
-SelectPolyRemove(Widget w, void *p)
-{
-    LocalInfo *l = (LocalInfo *) p;
-
-    OpRemoveEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
-			 (OpEventProc) pressPolyBand, l);
-    OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
-			 (OpEventProc) motionPolyBand, l);
-    OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
-			 (OpEventProc) releasePolyBand, l);
-    OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
-			 (OpEventProc) keyPress, l);
-    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
-			 (OpEventProc) keyRelease, l);
-    PwRegionFinish(w, False);
-
-    XtFree((XtPointer) l->window_p);
-    XtFree((XtPointer) l->real_p);
-    XtFree((XtPointer) l);
-}
-
-void 
-SelectSetShapeMode(int value)
-{
-    shapeMode = value;
-}
-
 void 
 SelectSetCutMode(int value)
 {
