@@ -13,7 +13,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: splineOp.c,v 1.2 2000/10/02 21:06:44 torsten Exp $ */
+/* $Id: splineOp.c,v 1.15 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <math.h>
 #include <stdlib.h>
@@ -26,7 +26,6 @@
 #include "Paint.h"
 #include "ops.h"
 
-#define MAXPOINTS 198
 #define SUBDIV 50
 #define FLOAT float
 
@@ -37,13 +36,14 @@ static int Mode = 0;
 static int ModeCtrl;
 static int ModeShift;
 static int Filled = 0;
+static int MAXPOINTS = 30;
 
 typedef struct {
     int zoom, rx, ry, mode, npoints, nprev;
-    XPoint points[MAXPOINTS + 2];
-    XPoint interm1[MAXPOINTS + 2];
-    XPoint interm2[MAXPOINTS + 2];
-    Boolean breakpt[MAXPOINTS + 2];
+    XPoint *points;
+    XPoint *interm1;
+    XPoint *interm2;
+    Boolean *breakpt;
     /*
     **  Borrowed from my info structure.
      */
@@ -62,11 +62,11 @@ static void
 XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
 {
     Display *dpy;
+    GC gc;
+    XPoint *xp;
     int n, i, j, imax, i0, i1=0, zoom, rx, ry;
     int u, v, up, vp;
     FLOAT ax=0.0, ay=0.0, bx, by, cx, cy, r=0.0, s, t;
-    XPoint *xp;
-    GC gc;
 
     dpy = XtDisplay(w);
 
@@ -216,13 +216,17 @@ XDrawContour(Widget w, Drawable d, LocalInfo *l, int flag)
 		    UndoGrow(w, xp[n].x, xp[n].y);
 	    }
 	}
-	if (Filled) {
+	if (Filled==1)
 	    XFillPolygon(dpy, d, l->sgc, xp, n+1, Complex, CoordModeOrigin);
-	    if (l->mode == OPEN) 
+	if (Filled && l->mode == OPEN) 
 	        xp[++n] = xp[0];
+        if (Filled<=1)
+	    XDrawLines(dpy, d, l->fgc, xp, n+1, CoordModeOrigin);
+	if (Filled==2 && d!=XtWindow(w)) {
+	    CreatePolygonalRegion(w, xp, n);
+	    free(xp);
+            return;
 	}
-	XDrawLines(dpy, d, l->fgc, xp, n+1, CoordModeOrigin);
-	free(xp);
     } else
     for (i = 0; i < imax; i++) {
         if ((i <= i0) || (i >= i1)) {
@@ -294,14 +298,19 @@ check_modifiers(XButtonEvent * event)
 }
 
 static void 
-press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
+pressSplineBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
+    PaintWidget pw = (PaintWidget) w;
+
     if (event->button == Button3) return;
     check_modifiers(event);
 
     if ((l->npoints < 0) && (event->button == Button1)) {
+      if (Filled==2 && pw->paint.region.isVisible) {
+	  l->npoints = -2;
+          return;
+      }
       if (info->surface == opPixmap) {
-
 	l->isFat = info->isFat;
 	l->fgc = info->first_gc;
 	l->sgc = info->second_gc;
@@ -327,17 +336,35 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     }
 }
 
-static void 
-release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
+void 
+releaseSplineBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
+    PaintWidget pw = (PaintWidget) w;
+
+    if (l->npoints==-2 && Filled==2 && pw->paint.region.isVisible) {
+        PwRegionFinish(w, True);
+        pw->paint.region.isVisible = False;
+	l->npoints = -1;
+	l->nprev = -1;
+        return;
+    }
 
     if (event->button == Button3) return;
 
     if (l->npoints < 0) 
         return;
 
-    if (l->npoints == MAXPOINTS - 1)
-	return;
+    if (l->npoints >= MAXPOINTS - 3) {
+        MAXPOINTS += 10;
+        l->points = 
+           (XPoint *) realloc(l->points, MAXPOINTS*sizeof(XPoint));
+        l->interm1 = 
+           (XPoint *) realloc(l->interm1, MAXPOINTS*sizeof(XPoint));
+        l->interm2 = 
+           (XPoint *) realloc(l->interm2, MAXPOINTS*sizeof(XPoint));
+        l->breakpt = 
+           (Boolean *) realloc(l->breakpt, MAXPOINTS*sizeof(Boolean));
+    }
 
     if ((event->button == Button1) && (info->surface == opWindow)) {
         check_modifiers(event);
@@ -350,8 +377,8 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     }
 }
 
-static void 
-motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
+void 
+motionSplineBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
     /*
     **  Haven't done the first button press
@@ -370,11 +397,34 @@ motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
     }
 }
 
+static
+LocalInfo * createLocalInfo()
+{
+    LocalInfo * l;
+    l = (LocalInfo *) malloc(sizeof(LocalInfo));
+    l->points =  (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
+    l->interm1 = (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
+    l->interm2 = (XPoint *) malloc(MAXPOINTS*sizeof(XPoint));
+    l->breakpt = (Boolean *) malloc(MAXPOINTS*sizeof(Boolean));
+    return l;
+}
+
+static 
+void freeLocalInfo(LocalInfo *l)
+{
+    free((void *)l->points);
+    free((void *) l->interm1);
+    free((void *) l->interm2);
+    free((void *) l->breakpt);
+    free((void *) l);
+}
+
 void *
 SplineAdd(Widget w)
 {
-    LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
-
+    LocalInfo *l;
+    
+    l = createLocalInfo();
     Filled = 0;
     ModeCtrl = Mode;
     l->npoints = -1;
@@ -384,11 +434,11 @@ SplineAdd(Widget w)
     XtVaSetValues(w, XtNcompress, True, NULL);
 
     OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
-		      (OpEventProc) press, l);
+		      (OpEventProc) pressSplineBand, l);
     OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
-		      (OpEventProc) motion, l);
+		      (OpEventProc) motionSplineBand, l);
     OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
-		      (OpEventProc) release, l);
+		      (OpEventProc) releaseSplineBand, l);
     SetCrossHairCursor(w);
 
     return l;
@@ -398,21 +448,22 @@ void
 SplineRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
-			 (OpEventProc) press, l);
+			 (OpEventProc) pressSplineBand, l);
     OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
-			 (OpEventProc) motion, l);
+			 (OpEventProc) motionSplineBand, l);
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
-			 (OpEventProc) release, l);
+			 (OpEventProc) releaseSplineBand, l);
 
     finish(w, (LocalInfo *) l);
-    XtFree((XtPointer) l);
+    freeLocalInfo((LocalInfo *) l);
 }
 
 void *
-FSplineAdd(Widget w)
+FilledSplineAdd(Widget w)
 {
-    LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
+    LocalInfo *l;
 
+    l = createLocalInfo();
     Filled = 1;
     ModeCtrl = Mode;
     l->npoints = -1;
@@ -422,26 +473,74 @@ FSplineAdd(Widget w)
     XtVaSetValues(w, XtNcompress, True, NULL);
 
     OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
-		      (OpEventProc) press, l);
+		      (OpEventProc) pressSplineBand, l);
     OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
-		      (OpEventProc) motion, l);
+		      (OpEventProc) motionSplineBand, l);
     OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
-		      (OpEventProc) release, l);
+		      (OpEventProc) releaseSplineBand, l);
     SetCrossHairCursor(w);
 
     return l;
 }
 
 void 
-FSplineRemove(Widget w, void *l)
+FilledSplineRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
-			 (OpEventProc) press, l);
+			 (OpEventProc) pressSplineBand, l);
     OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
-			 (OpEventProc) motion, l);
+			 (OpEventProc) motionSplineBand, l);
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
-			 (OpEventProc) release, l);
+			 (OpEventProc) releaseSplineBand, l);
 
     finish(w, (LocalInfo *) l);
-    XtFree((XtPointer) l);
+    freeLocalInfo((LocalInfo *) l);
+}
+
+void *
+SelectSplineAdd(Widget w)
+{
+    LocalInfo *l;
+
+    l = createLocalInfo();
+    Filled = 2;
+    ModeCtrl = Mode;
+    l->npoints = -1;
+    l->nprev = -1;
+    l->gcx = GetGCX(w);
+
+    XtVaSetValues(w, XtNcompress, True, NULL);
+
+    OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
+		      (OpEventProc) pressSplineBand, l);
+    OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
+		      (OpEventProc) motionSplineBand, l);
+    OpAddEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
+		      (OpEventProc) releaseSplineBand, l);
+    OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
+		      (OpEventProc) selectKeyPress, l);
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) selectKeyRelease, l);
+
+    SetCrossHairCursor(w);
+
+    return l;
+}
+
+void 
+SelectSplineRemove(Widget w, void *l)
+{
+    OpRemoveEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
+			 (OpEventProc) pressSplineBand, l);
+    OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
+			 (OpEventProc) motionSplineBand, l);
+    OpRemoveEventHandler(w, opWindow | opPixmap, ButtonReleaseMask, FALSE,
+			 (OpEventProc) releaseSplineBand, l);
+    OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
+		         (OpEventProc) selectKeyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		         (OpEventProc) selectKeyRelease, l);
+
+    finish(w, (LocalInfo *) l);
+    freeLocalInfo((LocalInfo *) l);
 }
